@@ -1,9 +1,9 @@
 from constants import *
 __title__ = APP_NAME
-__version__ = '0.10.8'
-__author__ = 'Zachary Huff'
+__version__ = '0.10.9'
+__author__ = 'Pritunl'
 __license__ = 'AGPL'
-__copyright__ = 'Copyright 2013-2014 Zachary Huff'
+__copyright__ = 'Copyright 2013-2014 Pritunl'
 
 from app_server import AppServer
 app_server = AppServer()
