@@ -1,5 +1,6 @@
 from constants import *
 from cache import cache_db, persist_db
+from event import Event
 from config import Config
 import utils
 import flask
@@ -21,15 +22,18 @@ logger = None
 class AppServer(Config):
     # deprecated = password, dh_param_bits
     bool_options = {'debug', 'log_debug', 'auto_start_servers',
-        'get_public_ip', 'inline_certs', 'ssl', 'static_cache'}
+        'get_public_ip', 'get_notifications', 'inline_certs', 'ssl',
+        'static_cache'}
     int_options = {'port', 'session_timeout', 'key_bits', 'dh_param_bits'}
     path_options = {'log_path', 'db_path', 'www_path', 'data_path',
         'server_cert_path', 'server_key_path'}
-    str_options = {'bind_addr', 'password', 'public_ip_server'}
+    str_options = {'bind_addr', 'password', 'public_ip_server',
+        'notification_server'}
     default_options = {
         'auto_start_servers': True,
         'get_public_ip': True,
         'inline_certs': True,
+        'get_notifications': True,
         'ssl': True,
         'static_cache': True,
         'bind_addr': DEFAULT_BIND_ADDR,
@@ -41,6 +45,7 @@ class AppServer(Config):
         'www_path': DEFAULT_WWW_PATH,
         'data_path': DEFAULT_DATA_PATH,
         'public_ip_server': DEFAULT_PUBLIC_IP_SERVER,
+        'notification_server': DEFAULT_NOTIFICATION_SERVER,
     }
     read_env = True
 
@@ -50,6 +55,15 @@ class AppServer(Config):
         self.interrupt = False
         self.public_ip = None
         self.conf_path = DEFAULT_CONF_PATH
+        self.notification = ''
+        self.www_state = OK
+        self.vpn_state = OK
+        self.sub_active = False
+        self.sub_status = None
+        self.sub_amount = None
+        self.sub_period_end = None
+        self.sub_cancel_at_period_end = None
+        self.openssl_heartbleed = not utils.check_openssl()
 
     def __getattr__(self, name):
         if name == 'web_protocol':
@@ -93,12 +107,83 @@ class AppServer(Config):
             except:
                 logger.exception('Failed to get public ip address...')
 
+    def subscription_update(self):
+        cur_sub_active = self.sub_active
+        license = persist_db.get('license')
+        if not license:
+            self.sub_active = False
+            self.sub_status = None
+            self.sub_amount = None
+            self.sub_period_end = None
+            self.sub_cancel_at_period_end = None
+        else:
+            try:
+                response = utils.request.get(SUBSCRIPTION_SERVER,
+                    json_data={'license': license},
+                    timeout=HTTP_REQUEST_TIMEOUT)
+                # License key invalid
+                if response.status_code == 470:
+                    persist_db.remove('license')
+                    self.subscription_update()
+                    return
+                data = response.json()
+            except:
+                logger.exception('Failed to check subscription status...')
+                data = {}
+            self.sub_active = data.get('active', True)
+            self.sub_status = data.get('status', 'unknown')
+            self.sub_amount = data.get('amount')
+            self.sub_period_end = data.get('period_end')
+            self.sub_cancel_at_period_end = data.get('cancel_at_period_end')
+        if cur_sub_active is not None and cur_sub_active != self.sub_active:
+            if self.sub_active:
+                Event(type=SUBSCRIPTION_ACTIVE)
+            else:
+                Event(type=SUBSCRIPTION_INACTIVE)
+
+    def subscription_dict(self):
+        return {
+            'license': bool(persist_db.get('license')),
+            'active': self.sub_active,
+            'status': self.sub_status,
+            'amount': self.sub_amount,
+            'period_end': self.sub_period_end,
+            'cancel_at_period_end': self.sub_cancel_at_period_end,
+        }
+
+    def _check_updates(self):
+        while True:
+            if self.get_notifications:
+                logger.debug('Checking notifications...')
+                try:
+                    request = urllib2.Request(self.notification_server + \
+                        '/%s' % self._get_version())
+                    response = urllib2.urlopen(request, timeout=60)
+                    data = json.load(response)
+
+                    self.notification = data.get('message', '')
+                    self.www_state = data.get('www', OK)
+                    self.vpn_state = data.get('vpn', OK)
+                except:
+                    logger.exception('Failed to check notifications.')
+            logger.debug('Checking subscription status...')
+            try:
+                self.subscription_update()
+            except:
+                logger.exception('Failed to check subscription status.')
+            time.sleep(UPDATE_CHECK_RATE)
+
     def _setup_public_ip(self):
         thread = threading.Thread(target=self.load_public_ip,
             kwargs={'attempts': 5})
         thread.daemon = True
         thread.start()
 
+    def _setup_updates(self):
+        thread = threading.Thread(target=self._check_updates)
+        thread.daemon = True
+        thread.start()
+
     def _setup_app(self):
         self.app = flask.Flask(APP_NAME)
         self.app.secret_key = os.urandom(32)
@@ -200,13 +285,6 @@ class AppServer(Config):
         version = self._get_version()
         cur_version = self._get_data_version()
 
-        if cur_version and cur_version < self._get_version_int('0.10.4'):
-            logger.info('Upgrading data to v0.10.4...')
-            from organization import Organization
-            for org in Organization.iter_orgs():
-                for user in org.iter_users():
-                    user._upgrade_0_10_4()
-
         if cur_version and cur_version < self._get_version_int('0.10.5'):
             logger.info('Upgrading data to v0.10.5...')
             from server import Server
@@ -222,7 +300,8 @@ class AppServer(Config):
             if self.password:
                 from cache import persist_db
                 logger.info('Upgrading config to v0.10.6...')
-                salt = base64.b64encode(PASSWORD_SALT_V0)
+                salt = base64.b64encode(
+                    '2511cebca93d028393735637bbc8029207731fcf')
                 password = base64.b64encode(self.password.decode('hex'))
                 persist_db.dict_set('auth', 'password',
                     '0$%s$%s' % (salt, password))
@@ -233,6 +312,18 @@ class AppServer(Config):
             for server in Server.iter_servers():
                 server._upgrade_0_10_6()
 
+        if cur_version and cur_version < self._get_version_int('0.10.9'):
+            logger.info('Upgrading data to v0.10.9...')
+            from server import Server
+            for server in Server.iter_servers():
+                server._upgrade_0_10_9()
+
+            from organization import Organization
+            for org in Organization.iter_orgs():
+                for user in org.iter_users():
+                    user._upgrade_0_10_9()
+                org.sort_users_cache()
+
         if cur_version != version:
             from pritunl import __version__
             version_path = os.path.join(self.data_path, VERSION_NAME)
@@ -254,6 +345,7 @@ class AppServer(Config):
         self._setup_conf()
         self._setup_log()
         self._setup_public_ip()
+        self._setup_updates()
         self._upgrade_db()
         self._setup_db()
         self._setup_handlers()
