@@ -9,6 +9,10 @@ class CacheTrie(object):
         self.name = name
         self.key = key
 
+    def clear_cache(self):
+        cache_db.remove(self.get_cache_key())
+        cache_db.remove(self.get_cache_key('values'))
+
     def get_cache_key(self, suffix=None):
         key = '%s_%s' % (self.name, self.key)
         if suffix is not None:
@@ -28,21 +32,21 @@ class CacheTrie(object):
         name = self.name + '_'
         cur_key = self.key
         new_key = cur_key
-        for char in key:
+        for char in key.lower():
             new_key += char
             cache_db.set_add(name + cur_key, new_key)
             cur_key = new_key
         cache_db.set_add(name + cur_key + '_values', value)
 
     def add_key_terms(self, key, value):
-        for term in re.split('[^a-zA-Z0-9]', key):
+        for term in re.split('[^a-z0-9]', key.lower()):
             self.add_key(term, value)
 
     def remove_key(self, key, value):
         name = self.name + '_'
         cur_key = self.key
         new_key = cur_key
-        for char in key:
+        for char in key.lower():
             new_key += char
             cache_db.set_remove(name + cur_key, new_key)
             cur_key = new_key
@@ -58,8 +62,8 @@ class CacheTrie(object):
         return values
 
     def get_prefix(self, prefix):
-        return CacheTrie(self.name, prefix).chain(set())
+        return CacheTrie(self.name, prefix.lower()).chain(set())
 
     def iter_prefix(self, prefix):
-        for value in CacheTrie(self.name, prefix).chain(set()):
+        for value in CacheTrie(self.name, prefix.lower()).chain(set()):
             yield value
