@@ -1,4 +1,5 @@
 import string
+import datetime
 
 APP_NAME = 'pritunl'
 APP_NAME_FORMATED = 'Pritunl'
@@ -11,6 +12,9 @@ START = 'start'
 STOP = 'stop'
 RESTART = 'restart'
 
+OK = 'ok'
+DISABLED = 'disabled'
+
 NAME_SAFE_CHARS = {'-', '=', '_', '@', '.', ':', '/'}
 
 KEY_LINK_TIMEOUT = 86400
@@ -20,7 +24,6 @@ DEFAULT_PRIMARY_INTERFACE = 'eth0'
 DEFAULT_SESSION_TIMEOUT = 86400
 DEFAULT_USERNAME = 'admin'
 DEFAULT_PASSWORD = 'admin'
-PASSWORD_SALT_V0 = '2511cebca93d028393735637bbc8029207731fcf'
 PASSWORD_LEN_LIMIT = 128
 DEFAULT_CONF_PATH = '/etc/pritunl.conf'
 DEFAULT_DB_PATH = '/var/lib/pritunl/pritunl.db'
@@ -30,6 +33,9 @@ DEFAULT_KEY_BITS = 4096
 DEFAULT_DH_PARAM_BITS = 1536
 DEFAULT_OTP_SECRET_LEN = 16
 DEFAULT_PUBLIC_IP_SERVER = 'http://ip.pritunl.com/json'
+DEFAULT_NOTIFICATION_SERVER = 'http://ip.pritunl.com/notification'
+SUBSCRIPTION_SERVER = 'https://app.pritunl.com/subscription'
+UPDATE_CHECK_RATE = 3600
 ENV_PREFIX = APP_NAME
 LOG_LIMIT = 100
 EVENT_TTL = 60
@@ -49,8 +55,23 @@ NODE_SERVER_VER = 1
 CLIENT_CONF_VER = 1
 STATIC_CACHE_TIME = 43200
 LOCALHOST_IP_TTL = 30
+AUTH_SIG_STRING_MAX_LEN = 10240
+AUTH_TIME_WINDOW = 300
 IP_REGEX = r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}'
 SAFE_PUB_SUBNETS = {'50.203.224.0/24'}
+STATIC_FILE_EXTENSIONS = {
+    '.css',
+    '.eot',
+    '.html',
+    '.ico',
+    '.js',
+    '.less',
+    '.png',
+    '.svg',
+    '.ttf',
+    '.txt',
+    '.woff',
+}
 
 INFO = 'info'
 WARNING = 'warning'
@@ -76,17 +97,23 @@ CLIENT_DISCONNECT_NAME = 'client_disconnect.py'
 OVPN_CONF_NAME = 'openvpn.conf'
 OVPN_STATUS_NAME = 'status'
 OVPN_CA_NAME = 'ca.crt'
-IFC_POOL_NAME = 'ifc_pool'
 DH_PARAM_NAME = 'dh_param.pem'
+IP_POOL_NAME = 'ip_pool'
 SERVER_USER_PREFIX = 'server_'
 SERVER_CERT_NAME = 'server.crt'
 SERVER_KEY_NAME = 'server.key'
+SERVER_CONF_NAME = 'server.conf'
 AUTH_LOG_NAME = 'auth.log'
 KEY_INDEX_NAME = 'key_index.html'
 CONF_TEMP_EXT = '.tmp'
+
 SERVER = 'server'
 NODE_SERVER = 'node_server'
 
+ALL_TRAFFIC = 'all_traffic'
+LOCAL_TRAFFIC = 'local_traffic'
+VPN_TRAFFIC = 'vpn_traffic'
+
 CA_CERT_ID = 'ca'
 CERT_CA = 'ca'
 CERT_SERVER = 'server'
@@ -98,6 +125,43 @@ LOG_UPDATED = 'log_updated'
 SERVERS_UPDATED = 'servers_updated'
 SERVER_ORGS_UPDATED = 'server_organizations_updated'
 SERVER_OUTPUT_UPDATED = 'server_output_updated'
+SUBSCRIPTION_ACTIVE = 'subscription_active'
+SUBSCRIPTION_INACTIVE = 'subscription_inactive'
+
+VALID_IP_ENDPOINTS = {
+    '5', '9', '13', '17', '21', '25', '29', '33', '37', '41', '45', '49',
+    '53', '57', '61', '65', '69', '73', '77', '81', '85', '89', '93', '97',
+    '101', '105', '109', '113', '117', '121', '125', '129', '133', '137',
+    '141', '145', '149', '153', '157', '161', '165', '169', '173', '177',
+    '181', '185', '189', '193', '197', '201', '205', '209', '213', '217',
+    '221', '225', '229', '233', '237', '241', '245', '249', '253'
+}
+
+OPENSSL_HEARTBLEED = {
+    'OpenSSL 1.0.1-fips-beta1 03 Jan 2012',
+    'OpenSSL 1.0.1-beta1 03 Jan 2012',
+    'OpenSSL 1.0.1-fips-beta2 19 Jan 2012',
+    'OpenSSL 1.0.1-beta2 19 Jan 2012',
+    'OpenSSL 1.0.1-fips-beta3 23 Feb 2012',
+    'OpenSSL 1.0.1-beta3 23 Feb 2012',
+    'OpenSSL 1.0.1-fips 14 Mar 2012',
+    'OpenSSL 1.0.1 14 Mar 2012',
+    'OpenSSL 1.0.1a-fips 19 Apr 2012',
+    'OpenSSL 1.0.1a 19 Apr 2012',
+    'OpenSSL 1.0.1b-fips 26 Apr 2012',
+    'OpenSSL 1.0.1b 26 Apr 2012',
+    'OpenSSL 1.0.1c-fips 10 May 2012',
+    'OpenSSL 1.0.1c 10 May 2012',
+    'OpenSSL 1.0.1d-fips 5 Feb 2013',
+    'OpenSSL 1.0.1d 5 Feb 2013',
+    'OpenSSL 1.0.1e-fips 11 Feb 2013',
+    'OpenSSL 1.0.1e 11 Feb 2013',
+    'OpenSSL 1.0.1f-fips 6 Jan 2014',
+    'OpenSSL 1.0.1f 6 Jan 2014',
+    'OpenSSL 1.0.2-beta1-fips 24 Feb 2014',
+    'OpenSSL 1.0.2-beta1 24 Feb 2014',
+}
+OPENSSL_HEARTBLEED_BUILD_DATE = datetime.date(2014, 4, 7)
 
 CERT_CONF = """[ default ]
 ca = %s
@@ -202,6 +266,10 @@ DH_PARAM_BITS_INVALID = 'dh_param_bits_invalid'
 DH_PARAM_BITS_INVALID_MSG = 'DH param bits are not valid, must ' + \
     '1024, 1536, 2048, 2048, 3072 or 4096.'
 
+MODE_INVALID = 'mode_invalid'
+MODE_INVALID_MSG = 'Mode is not valid, must be "all_traffic" or ' + \
+    '"local_traffic" or "vpn_traffic".'
+
 INTERFACE_INVALID = 'interface_invalid'
 INTERFACE_INVALID_MSG = 'Interface is not valid, must be ' + \
     '"tun[0-64]" example "tun0".'
@@ -240,9 +308,16 @@ ORG_INVALID_MSG = 'Organization is not valid.'
 USER_INVALID = 'user_invalid'
 USER_INVALID_MSG = 'User is not valid.'
 
+USER_TYPE_INVALID = 'user_type_invalid'
+USER_TYPE_INVALID_MSG = 'User type is not valid.'
+
 OTP_CODE_INVALID = 'otp_code_invalid'
 OTP_CODE_INVALID_MSG = 'OTP code is not valid.'
 
+SUBSCRIPTION_SERVER_ERROR = 'subscription_server_error'
+SUBSCRIPTION_SERVER_ERROR_MSG = 'Unable to connect to ' + \
+    'subscription server, please try again later.'
+
 OVPN_SERVER_CONF = """port %s
 proto %s
 dev %s
@@ -254,10 +329,9 @@ client-connect %s
 client-disconnect %s
 dh %s
 server %s
-ifconfig-pool-persist %s 604800
 %s
 max-clients 1024
-keepalive 10 30
+keepalive 10 %s
 persist-tun
 status %s 1
 status-version 2
@@ -273,10 +347,9 @@ tls-verify %s
 client-connect %s
 client-disconnect %s
 server %s
-ifconfig-pool-persist %s 604800
 %s
 max-clients 1024
-keepalive 10 30
+keepalive 10 %s
 persist-tun
 status %s 1
 status-version 2
@@ -371,7 +444,7 @@ try:
         }).encode('utf-8'))
         response = json.loads(response.read().decode('utf-8'))
 
-        if not response['authenticated']:
+        if not response.get('authenticated'):
             log_write('[FAILED] Invalid user id or organization id')
             exit(1)
     except HTTPError as error:
@@ -464,7 +537,7 @@ try:
         }).encode('utf-8'))
         response = json.loads(response.read().decode('utf-8'))
 
-        if not response['authenticated']:
+        if not response.get('authenticated'):
             log_write('[FAILED] Invalid user id or organization id')
             exit(1)
     except HTTPError as error:
