@@ -85,10 +85,8 @@ class NodeServer(Server):
             'connection occurred "%s".' % self.name)
 
     def _start_server_threads(self):
-        thread_data = {
-            'state': True,
-            'ws': None,
-        }
+        self._state = True
+        self._ws = None
 
         def com_thread():
             ws = websocket.WebSocketApp(
@@ -97,13 +95,14 @@ class NodeServer(Server):
                 on_message=self._com_on_message,
                 on_error=self._com_on_error,
             )
-            thread_data['ws'] = ws
+            self._ws = ws
             ws.run_forever(ping_interval=SOCKET_PING_INTERVAL,
                 timeout=SOCKET_TIMEOUT)
             self.status = False
             self.publish('stopped')
+            self.update_clients({}, force=True)
 
-            if thread_data['state']:
+            if self._state:
                 LogEntry(message='Node server stopped unexpectedly "%s".' % (
                     self.name))
                 Event(type=SERVERS_UPDATED)
@@ -112,10 +111,9 @@ class NodeServer(Server):
             for message in cache_db.subscribe(self.get_cache_key()):
                 try:
                     if message == 'stop':
-                        thread_data['state'] = False
-                        ws = thread_data.get('ws')
-                        if ws:
-                            ws.close()
+                        self._state = False
+                        if self._ws:
+                            self._ws.close()
                     elif message == 'stopped':
                         break
                 except OSError:
@@ -202,14 +200,23 @@ class NodeServer(Server):
             self._generate_dh_param()
 
         primary_org = Organization.get_org(id=self.primary_organization)
+        if not primary_org:
+            self._create_primary_user()
+        primary_org = Organization.get_org(id=self.primary_organization)
+
+        primary_user = primary_org.get_user(self.primary_user)
+        if not primary_user:
+            self._create_primary_user()
         primary_user = primary_org.get_user(self.primary_user)
 
         self.generate_ca_cert()
 
         push = ''
-        if self.local_networks:
+        if self.mode == LOCAL_TRAFFIC:
             for network in self.local_networks:
                 push += 'push "route %s %s"\n' % self._parse_network(network)
+        elif self.mode == VPN_TRAFFIC:
+            pass
         else:
             push += 'push "redirect-gateway"\n'
         for dns_server in self.dns_servers:
@@ -224,8 +231,8 @@ class NodeServer(Server):
             '%s',
             '%s',
             '%s %s' % self._parse_network(self.network),
-            '%s',
             push,
+            '120' if self.otp_auth else '60',
             '%s',
             4 if self.debug else 1,
             8 if self.debug else 3,
@@ -253,76 +260,84 @@ class NodeServer(Server):
         return server_conf
 
     def start(self, silent=False):
-        if self.status:
-            return
-        if not self.org_count:
-            raise ServerMissingOrg('Server cannot be started without ' + \
-                'any organizations', {
+        cache_db.lock_acquire(self.get_cache_key('op_lock'))
+        try:
+            if self.status:
+                return
+            if not self.org_count:
+                raise ServerMissingOrg('Server cannot be started without ' + \
+                    'any organizations', {
+                        'server_id': self.id,
+                    })
+
+            logger.debug('Starting node server. %r' % {
+                'server_id': self.id,
+            })
+            ovpn_conf = self._generate_ovpn_conf()
+
+            try:
+                response = self._request('post', json_data={
+                    'interface': self.interface,
+                    'network': self.network,
+                    'local_networks': self.local_networks,
+                    'ovpn_conf': ovpn_conf,
+                    'server_ver': NODE_SERVER_VER,
+                })
+            except httplib.HTTPException:
+                raise NodeConnectionError('Failed to connect to node server', {
                     'server_id': self.id,
                 })
 
-        logger.debug('Starting node server. %r' % {
-            'server_id': self.id,
-        })
-        ovpn_conf = self._generate_ovpn_conf()
+            if response.status_code == 401:
+                raise InvalidNodeAPIKey('Invalid node server api key', {
+                    'server_id': self.id,
+                    'status_code': response.status_code,
+                    'reason': response.reason,
+                })
+            elif response.status_code != 200:
+                raise ServerStartError('Failed to start node server', {
+                    'server_id': self.id,
+                    'status_code': response.status_code,
+                    'reason': response.reason,
+                })
 
-        try:
-            response = self._request('post', json_data={
-                'interface': self.interface,
-                'network': self.network,
-                'local_networks': self.local_networks,
-                'ovpn_conf': ovpn_conf,
-                'server_ver': NODE_SERVER_VER,
-            })
-        except httplib.HTTPException:
-            raise NodeConnectionError('Failed to connect to node server', {
-                'server_id': self.id,
-            })
+            cache_db.dict_set(self.get_cache_key(), 'start_time',
+                str(int(time.time() - 1)))
+            self.clear_output()
+            self._interrupt = False
+            self.status = True
+            self._start_server_threads()
 
-        if response.status_code == 401:
-            raise InvalidNodeAPIKey('Invalid node server api key', {
-                'server_id': self.id,
-                'status_code': response.status_code,
-                'reason': response.reason,
-            })
-        elif response.status_code != 200:
-            raise ServerStartError('Failed to start node server', {
-                'server_id': self.id,
-                'status_code': response.status_code,
-                'reason': response.reason,
-            })
-
-        cache_db.dict_set(self.get_cache_key(), 'start_time',
-            str(int(time.time() - 1)))
-        self.clear_output()
-        self._interrupt = False
-        self.status = True
-        self._start_server_threads()
-
-        if not silent:
-            Event(type=SERVERS_UPDATED)
-            LogEntry(message='Started server "%s".' % self.name)
+            if not silent:
+                Event(type=SERVERS_UPDATED)
+                LogEntry(message='Started server "%s".' % self.name)
+        finally:
+            cache_db.lock_release(self.get_cache_key('op_lock'))
 
     def stop(self, silent=False):
-        if not self.status:
-            return
+        cache_db.lock_acquire(self.get_cache_key('op_lock'))
+        try:
+            if not self.status:
+                return
 
-        stopped = False
-        self.publish('stop')
-        for message in cache_db.subscribe(self.get_cache_key(),
-                SUB_RESPONSE_TIMEOUT):
-            if message == 'stopped':
-                stopped = True
-                break
-        if not stopped:
-            raise ServerStopError('Server thread failed to return ' + \
-                'stop event', {
-                    'server_id': self.id,
-                })
+            stopped = False
+            self.publish('stop')
+            for message in cache_db.subscribe(self.get_cache_key(),
+                    SUB_RESPONSE_TIMEOUT):
+                if message == 'stopped':
+                    stopped = True
+                    break
+            if not stopped:
+                raise ServerStopError('Server thread failed to return ' + \
+                    'stop event', {
+                        'server_id': self.id,
+                    })
 
-        if not silent:
-            Event(type=SERVERS_UPDATED)
-            LogEntry(message='Stopped server "%s".' % self.name)
+            if not silent:
+                Event(type=SERVERS_UPDATED)
+                LogEntry(message='Stopped server "%s".' % self.name)
+        finally:
+            cache_db.lock_release(self.get_cache_key('op_lock'))
 
     def force_stop(self, silent=False):
         self.stop(silent)
