@@ -78,12 +78,18 @@ class Organization(Config):
 
     def _initialize(self):
         self._make_dirs()
-        self.ca_cert = User(self, type=CERT_CA)
-        self.commit()
-        cache_db.set_add('orgs', self.id)
-        self.sort_orgs_cache()
-        LogEntry(message='Created new organization "%s".' % self.name)
-        Event(type=ORGS_UPDATED)
+        try:
+            self.ca_cert = User(self, type=CERT_CA)
+            cache_db.set_add('orgs', self.id)
+            self.commit()
+            LogEntry(message='Created new organization "%s".' % self.name)
+        except:
+            logger.exception('Failed to create organization. %r' % {
+                'org_id': self.id,
+            })
+            self.clear_cache()
+            utils.rmtree(self.path)
+            raise
 
     def _make_dirs(self):
         os.makedirs(os.path.join(self.path, REQS_DIR))
@@ -98,6 +104,7 @@ class Organization(Config):
         cache_db.decrement('org_count')
         cache_db.remove(self.get_cache_key('users_cached'))
         cache_db.remove(self.get_cache_key('users'))
+        CacheTrie(self.get_cache_key('users_trie')).clear_cache()
         Config.clear_cache(self)
 
     def get_user(self, id):
@@ -158,7 +165,6 @@ class Organization(Config):
 
     def _cache_users(self):
         if cache_db.get(self.get_cache_key('users_cached')) != 't':
-            users_trie = CacheTrie(self.get_cache_key('users_trie'))
             cache_db.remove(self.get_cache_key('users'))
             certs_path = os.path.join(self.path, CERTS_DIR)
             if os.path.isdir(certs_path):
@@ -169,7 +175,7 @@ class Organization(Config):
                     user = User.get_user(self, id=user_id)
                     if not user:
                         continue
-                    users_trie.add_key_terms(user.name, user_id)
+                    user._add_cache_trie_key()
                     cache_db.set_add(self.get_cache_key('users'), user_id)
             self.sort_users_cache()
             cache_db.set(self.get_cache_key('users_cached'), 't')
@@ -210,22 +216,25 @@ class Organization(Config):
             users_sort = []
             prefix_count = 0
             users_trie = CacheTrie(self.get_cache_key('users_trie'))
-            for user_id in users_trie.iter_prefix(prefix):
-                user = User.get_user(self, id=user_id)
-                if not user:
-                    continue
-                if user.type == CERT_CLIENT:
+
+            for user_data in users_trie.iter_prefix(prefix):
+                user_id, user_type, user_name = user_data.split('-', 2)
+                if user_type == CERT_CLIENT:
                     prefix_count += 1
-                name_id = '%s_%s' % (user.name, user_id)
-                users_dict[name_id] = user
+                name_id = user_name + '_' + user_id
+                users_dict[name_id] = (user_id, user_type, user_name)
                 users_sort.append(name_id)
             self._last_prefix_count = prefix_count
 
             user_count = 0
             search_more = False
             for name_id in sorted(users_sort):
-                yield users_dict[name_id]
-                if prefix_limit and users_dict[name_id].type == CERT_CLIENT:
+                user_id, user_type, user_name = users_dict[name_id]
+                user = User.get_user(self, id=user_id)
+                if not user:
+                    continue
+                yield user
+                if prefix_limit and user_type == CERT_CLIENT:
                     user_count += 1
                     if user_count >= prefix_limit:
                         search_more = True
@@ -260,8 +269,6 @@ class Organization(Config):
     def rename(self, name):
         self.name = name
         self.commit()
-        self.sort_orgs_cache()
-        Event(type=ORGS_UPDATED)
 
     def remove(self):
         self.clear_cache()
@@ -276,6 +283,11 @@ class Organization(Config):
         LogEntry(message='Deleted organization "%s".' % name)
         Event(type=ORGS_UPDATED)
 
+    def commit(self):
+        Config.commit(self)
+        self.sort_orgs_cache()
+        Event(type=ORGS_UPDATED)
+
     @classmethod
     def get_org(cls, id):
         org = Organization(id=id)
@@ -311,7 +323,7 @@ class Organization(Config):
                 cache_db.list_rpush(temp_orgs_sorted_key, orgs_dict[name_id])
             cache_db.rename(temp_orgs_sorted_key, 'orgs_sorted')
         except:
-            cache_db.remove(self.get_cache_key(temp_orgs_sorted_key))
+            cache_db.remove(temp_orgs_sorted_key)
             raise
 
     @classmethod
