@@ -18,11 +18,12 @@ import traceback
 import utils
 import re
 import json
+import ipaddress
 
 logger = logging.getLogger(APP_NAME)
 
 class Server(Config):
-    str_options = {'name', 'network', 'interface', 'protocol',
+    str_options = {'name', 'network', 'interface', 'protocol', 'mode',
         'local_networks', 'public_address', 'primary_organization',
         'primary_user', 'organizations', 'local_network', 'dns_servers'}
     bool_options = {'otp_auth', 'lzo_compression', 'debug'}
@@ -37,6 +38,7 @@ class Server(Config):
         self._cur_event = None
         self._last_event = 0
         self._rebuild_dh_params = False
+        self._reset_ip_pool = False
 
         if id is None:
             self.id = uuid.uuid4().hex
@@ -48,7 +50,7 @@ class Server(Config):
         self.path = os.path.join(app_server.data_path, SERVERS_DIR, self.id)
         self.ovpn_conf_path = os.path.join(self.path, TEMP_DIR, OVPN_CONF_NAME)
         self.dh_param_path = os.path.join(self.path, DH_PARAM_NAME)
-        self.ifc_pool_path = os.path.join(self.path, IFC_POOL_NAME)
+        self.ip_pool_path = os.path.join(self.path, IP_POOL_NAME)
         self.ca_cert_path = os.path.join(self.path, TEMP_DIR, OVPN_CA_NAME)
         self.tls_verify_path = os.path.join(self.path, TEMP_DIR,
             TLS_VERIFY_NAME)
@@ -61,17 +63,15 @@ class Server(Config):
         self.ovpn_status_path = os.path.join(self.path, TEMP_DIR,
             OVPN_STATUS_NAME)
         self.auth_log_path = os.path.join(app_server.data_path, AUTH_LOG_NAME)
-        self.set_path(os.path.join(self.path, 'server.conf'))
+        self.set_path(os.path.join(self.path, SERVER_CONF_NAME))
 
         if id is None:
             self._initialize()
 
     def __setattr__(self, name, value):
         if name == 'status':
-            if value:
-                cache_db.dict_set(self.get_cache_key(), name, 't')
-            else:
-                cache_db.dict_set(self.get_cache_key(), name, 'f')
+            cache_db.dict_set(self.get_cache_key(), name,
+                't' if value else 'f')
             return
         elif name == 'clients':
             cache_db.dict_set(self.get_cache_key(), name, json.dumps(value))
@@ -79,13 +79,13 @@ class Server(Config):
         elif name == 'dh_param_bits':
             if not self._loaded or self.dh_param_bits != value:
                 self._rebuild_dh_params = True
+        elif name == 'network':
+            self._reset_ip_pool = self._loaded and self.network != value
         Config.__setattr__(self, name, value)
 
     def __getattr__(self, name):
         if name == 'status':
-            if cache_db.dict_get(self.get_cache_key(), name) == 't':
-                return True
-            return False
+            return cache_db.dict_get(self.get_cache_key(), name) == 't'
         elif name == 'uptime':
             if self.status:
                 return int(time.time()) - int(cache_db.dict_get(
@@ -119,6 +119,7 @@ class Server(Config):
             'port': self.port,
             'protocol': self.protocol,
             'dh_param_bits': self.dh_param_bits,
+            'mode': self.mode,
             'local_networks': self.local_networks,
             'dns_servers': self.dns_servers,
             'public_address': self.public_address,
@@ -128,6 +129,10 @@ class Server(Config):
             'org_count': self.org_count,
         }
 
+    def load(self, *args, **kwargs):
+        Config.load(self, *args, **kwargs)
+        self._load_ip_pool()
+
     def _upgrade_0_10_5(self):
         if self.local_network:
             logger.debug('Upgrading server to v0.10.5... %r' % {
@@ -145,6 +150,26 @@ class Server(Config):
             self.dh_param_bits = app_server.dh_param_bits
             self.commit()
 
+    def _upgrade_0_10_9(self):
+        if not self.mode:
+            logger.debug('Upgrading server to v0.10.9... %r' % {
+                'server_id': self.id,
+            })
+            if self.local_networks:
+                self.mode = LOCAL_TRAFFIC
+            else:
+                self.mode = VPN_TRAFFIC
+            self.commit()
+
+        self.update_ip_pool()
+
+        try:
+            ifc_pool_path = os.path.join(self.path, 'ifc_pool')
+            if os.path.exists(ifc_pool_path):
+                os.remove(ifc_pool_path)
+        except:
+            pass
+
     def _initialize(self):
         logger.debug('Initialize new server. %r' % {
             'server_id': self.id,
@@ -163,10 +188,102 @@ class Server(Config):
             utils.rmtree(self.path)
             raise
 
-    def clear_cache(self):
+    def _load_ip_pool(self):
+        if cache_db.get(self.get_cache_key('ip_pool_cached')) == 't':
+            return
+        reset = False
+
+        if os.path.exists(self.ip_pool_path):
+            with open(self.ip_pool_path, 'r') as ip_pool_file:
+                pool = json.loads(ip_pool_file.read())
+
+            network = pool.pop('network', None)
+            if network == self.network:
+                cache_key = self.get_cache_key('ip_pool')
+                set_cache_key = self.get_cache_key('ip_pool_set')
+                for key, value in pool.iteritems():
+                    cache_db.dict_set(cache_key, key, value)
+                    local_ip_addr, remote_ip_addr = value.split('-')
+                    cache_db.set_add(set_cache_key, local_ip_addr)
+                    cache_db.set_add(set_cache_key, remote_ip_addr)
+            else:
+                reset = True
+
+        cache_db.set(self.get_cache_key('ip_pool_cached'), 't')
+
+        if reset:
+            self.update_ip_pool()
+
+    def _commit_ip_pool(self):
+        with open(self.ip_pool_path, 'w') as ip_pool_file:
+            pool = cache_db.dict_get_all(self.get_cache_key('ip_pool'))
+            pool['network'] = self.network
+            ip_pool_file.write(json.dumps(pool))
+
+    def update_ip_pool(self):
+        cache_key = self.get_cache_key('ip_pool')
+        set_cache_key = self.get_cache_key('ip_pool_set')
+        cache_db.lock_acquire(cache_key)
+        try:
+            ip_pool = ipaddress.IPv4Network(self.network).iterhosts()
+            ip_pool.next()
+
+            users = set()
+            for org in self.iter_orgs():
+                for user in org.iter_users():
+                    if user.type == CERT_CLIENT:
+                        users.add(org.id + '-' + user.id)
+
+            for user_id in cache_db.dict_keys(cache_key) - users:
+                ip_set = cache_db.dict_get(cache_key, user_id)
+                local_ip_addr, remote_ip_addr = ip_set.split('-')
+                cache_db.set_remove(set_cache_key, local_ip_addr)
+                cache_db.set_remove(set_cache_key, remote_ip_addr)
+                cache_db.dict_remove(cache_key, user_id)
+
+            try:
+                for user_id in users - cache_db.dict_keys(cache_key):
+                    while True:
+                        remote_ip_addr = str(ip_pool.next())
+                        ip_addr_endpoint = remote_ip_addr.split('.')[-1]
+                        if ip_addr_endpoint not in VALID_IP_ENDPOINTS:
+                            continue
+                        local_ip_addr = str(ip_pool.next())
+
+                        if not cache_db.set_exists(set_cache_key,
+                                local_ip_addr) and not cache_db.set_exists(
+                                set_cache_key, remote_ip_addr):
+                            cache_db.set_add(set_cache_key, local_ip_addr)
+                            cache_db.set_add(set_cache_key, remote_ip_addr)
+                            break
+                    cache_db.dict_set(cache_key, user_id,
+                        local_ip_addr + '-' + remote_ip_addr)
+            except StopIteration:
+                pass
+            finally:
+                self._commit_ip_pool()
+                for org in self.iter_orgs():
+                    Event(type=USERS_UPDATED, resource_id=org.id)
+        finally:
+            cache_db.lock_release(cache_key)
+
+    def get_ip_set(self, org_id, user_id):
+        ip_set = cache_db.dict_get(self.get_cache_key('ip_pool'),
+            org_id + '-' + user_id)
+        if ip_set:
+            return ip_set.split('-')
+        return None, None
+
+    def _clear_list_cache(self):
         cache_db.set_remove('servers', '%s_%s' % (self.id, self.type))
         cache_db.list_remove('servers_sorted', '%s_%s' % (self.id, self.type))
+
+    def clear_cache(self):
+        self._clear_list_cache()
         cache_db.remove(self.get_cache_key('clients'))
+        cache_db.remove(self.get_cache_key('ip_pool'))
+        cache_db.remove(self.get_cache_key('ip_pool_set'))
+        cache_db.remove(self.get_cache_key('ip_pool_cached'))
         for period in ('1m', '5m', '30m', '2h', '1d'):
             persist_db.remove(self.get_cache_key('bandwidth-%s' % period))
         Config.clear_cache(self)
@@ -193,20 +310,28 @@ class Server(Config):
         logger.debug('Removing server. %r' % {
             'server_id': self.id,
         })
-        self.clear_cache()
+        self._clear_list_cache()
         name = self.name
+        orgs = list(self.iter_orgs())
 
         if self.status:
             self.force_stop(True)
+        self.clear_cache()
 
         self._remove_primary_user()
         utils.rmtree(self.path)
         LogEntry(message='Deleted server "%s".' % name)
         Event(type=SERVERS_UPDATED)
+        for org in orgs:
+            Event(type=USERS_UPDATED, resource_id=org.id)
 
     def commit(self):
         if self._rebuild_dh_params:
             self._generate_dh_param()
+        if self._reset_ip_pool:
+            cache_db.remove(self.get_cache_key('ip_pool'))
+            cache_db.remove(self.get_cache_key('ip_pool_set'))
+            self.update_ip_pool()
         Config.commit(self)
         self.sort_servers_cache()
         Event(type=SERVERS_UPDATED)
@@ -249,6 +374,7 @@ class Server(Config):
             return org
         self.organizations.append(org.id)
         self.commit()
+        self.update_ip_pool()
         Event(type=SERVERS_UPDATED)
         Event(type=SERVER_ORGS_UPDATED, resource_id=self.id)
         Event(type=USERS_UPDATED, resource_id=org_id)
@@ -289,6 +415,7 @@ class Server(Config):
             self._remove_primary_user()
         self.organizations.remove(org_id)
         self.commit()
+        self.update_ip_pool()
         Event(type=SERVERS_UPDATED)
         Event(type=SERVER_ORGS_UPDATED, resource_id=self.id)
         Event(type=USERS_UPDATED, resource_id=org_id)
@@ -309,9 +436,7 @@ class Server(Config):
             yield orgs_dict[name_id]
 
     def get_org(self, org_id):
-        for org_id in self.organizations:
-            if org_id != org_id:
-                continue
+        if org_id in self.organizations:
             org = Organization.get_org(id=org_id)
             try:
                 org.load()
@@ -403,15 +528,24 @@ class Server(Config):
             self._generate_dh_param()
 
         primary_org = Organization.get_org(id=self.primary_organization)
+        if not primary_org:
+            self._create_primary_user()
+        primary_org = Organization.get_org(id=self.primary_organization)
+
+        primary_user = primary_org.get_user(self.primary_user)
+        if not primary_user:
+            self._create_primary_user()
         primary_user = primary_org.get_user(self.primary_user)
 
         self.generate_ca_cert()
         self._generate_scripts()
 
         push = ''
-        if self.local_networks:
+        if self.mode == LOCAL_TRAFFIC:
             for network in self.local_networks:
                 push += 'push "route %s %s"\n' % self._parse_network(network)
+        elif self.mode == VPN_TRAFFIC:
+            pass
         else:
             push += 'push "redirect-gateway"\n'
         for dns_server in self.dns_servers:
@@ -431,8 +565,8 @@ class Server(Config):
                 self.client_disconnect_path,
                 self.dh_param_path,
                 '%s %s' % self._parse_network(self.network),
-                self.ifc_pool_path,
                 push,
+                '120' if self.otp_auth else '60',
                 self.ovpn_status_path,
                 4 if self.debug else 1,
                 8 if self.debug else 3,
@@ -446,8 +580,8 @@ class Server(Config):
                 self.client_connect_path,
                 self.client_disconnect_path,
                 '%s %s' % self._parse_network(self.network),
-                self.ifc_pool_path,
                 push,
+                '120' if self.otp_auth else '60',
                 self.ovpn_status_path,
                 4 if self.debug else 1,
                 8 if self.debug else 3,
@@ -460,7 +594,7 @@ class Server(Config):
         if self.lzo_compression:
             server_conf += 'comp-lzo\npush "comp-lzo"\n'
 
-        if self.local_networks:
+        if self.mode in (LOCAL_TRAFFIC, VPN_TRAFFIC):
             server_conf += 'client-to-client\n'
 
         if inline:
@@ -597,8 +731,10 @@ class Server(Config):
         for message in cache_db.subscribe(self.get_cache_key()):
             try:
                 if message == 'stop':
+                    self._state = False
                     process.send_signal(signal.SIGINT)
                 elif message == 'force_stop':
+                    self._state = False
                     process.send_signal(signal.SIGKILL)
                 elif message == 'stopped':
                     break
@@ -623,6 +759,7 @@ class Server(Config):
             'server_id': self.id,
         })
         self._interrupt = False
+        self._state = True
         try:
             try:
                 process = subprocess.Popen(['openvpn', self.ovpn_conf_path],
@@ -661,7 +798,8 @@ class Server(Config):
 
             self.status = False
             self.publish('stopped')
-            if process.returncode != 0:
+            self.update_clients({}, force=True)
+            if self._state:
                 Event(type=SERVERS_UPDATED)
                 LogEntry(message='Server stopped unexpectedly "%s".' % (
                     self.name))
@@ -678,90 +816,102 @@ class Server(Config):
         cache_db.publish(self.get_cache_key(), message)
 
     def start(self, silent=False):
-        if self.status:
-            return
-        if not self.org_count:
-            raise ServerMissingOrg('Server cannot be started without ' + \
-                'any organizations', {
-                    'server_id': self.id,
-                })
-        logger.debug('Starting server. %r' % {
-            'server_id': self.id,
-        })
-        self._generate_ovpn_conf()
-        self._enable_ip_forwarding()
-        self._set_iptables_rules()
-        self.clear_output()
+        cache_db.lock_acquire(self.get_cache_key('op_lock'))
+        try:
+            if self.status:
+                return
+            if not self.org_count:
+                raise ServerMissingOrg('Server cannot be started without ' + \
+                    'any organizations', {
+                        'server_id': self.id,
+                    })
+            logger.debug('Starting server. %r' % {
+                'server_id': self.id,
+            })
+            self._generate_ovpn_conf()
+            self._enable_ip_forwarding()
+            self._set_iptables_rules()
+            self.clear_output()
 
-        threading.Thread(target=self._run_thread).start()
+            threading.Thread(target=self._run_thread).start()
 
-        started = False
-        for message in cache_db.subscribe(self.get_cache_key(),
-                SUB_RESPONSE_TIMEOUT):
-            if message == 'started':
-                started = True
-                break
-            elif message == 'stopped':
-                raise ServerStartError('Server failed to start', {
-                    'server_id': self.id,
-                })
-        if not started:
-            raise ServerStartError('Server thread failed to return ' + \
-                'start event', {
-                    'server_id': self.id,
-                })
+            started = False
+            for message in cache_db.subscribe(self.get_cache_key(),
+                    SUB_RESPONSE_TIMEOUT):
+                if message == 'started':
+                    started = True
+                    break
+                elif message == 'stopped':
+                    raise ServerStartError('Server failed to start', {
+                        'server_id': self.id,
+                    })
+            if not started:
+                raise ServerStartError('Server thread failed to return ' + \
+                    'start event', {
+                        'server_id': self.id,
+                    })
 
-        if not silent:
-            Event(type=SERVERS_UPDATED)
-            LogEntry(message='Started server "%s".' % self.name)
+            if not silent:
+                Event(type=SERVERS_UPDATED)
+                LogEntry(message='Started server "%s".' % self.name)
+        finally:
+            cache_db.lock_release(self.get_cache_key('op_lock'))
 
     def stop(self, silent=False):
-        if not self.status:
-            return
-        logger.debug('Stopping server. %r' % {
-            'server_id': self.id,
-        })
+        cache_db.lock_acquire(self.get_cache_key('op_lock'))
+        try:
+            if not self.status:
+                return
+            logger.debug('Stopping server. %r' % {
+                'server_id': self.id,
+            })
 
-        stopped = False
-        self.publish('stop')
-        for message in cache_db.subscribe(self.get_cache_key(),
-                SUB_RESPONSE_TIMEOUT):
-            if message == 'stopped':
-                stopped = True
-                break
-        if not stopped:
-            raise ServerStopError('Server thread failed to return ' + \
-                'stop event', {
-                    'server_id': self.id,
-                })
+            stopped = False
+            self.publish('stop')
+            for message in cache_db.subscribe(self.get_cache_key(),
+                    SUB_RESPONSE_TIMEOUT):
+                if message == 'stopped':
+                    stopped = True
+                    break
+            if not stopped:
+                raise ServerStopError('Server thread failed to return ' + \
+                    'stop event', {
+                        'server_id': self.id,
+                    })
 
-        if not silent:
-            Event(type=SERVERS_UPDATED)
-            LogEntry(message='Stopped server "%s".' % self.name)
+            if not silent:
+                Event(type=SERVERS_UPDATED)
+                LogEntry(message='Stopped server "%s".' % self.name)
+        finally:
+            cache_db.lock_release(self.get_cache_key('op_lock'))
 
     def force_stop(self, silent=False):
-        if not self.status:
-            return
-        logger.debug('Forcing stop server. %r' % {
-            'server_id': self.id,
-        })
+        cache_db.lock_acquire(self.get_cache_key('op_lock'))
+        try:
+            if not self.status:
+                return
+            logger.debug('Forcing stop server. %r' % {
+                'server_id': self.id,
+            })
 
-        stopped = False
-        self.publish('force_stop')
-        for message in cache_db.subscribe(self.get_cache_key(),
-                SUB_RESPONSE_TIMEOUT):
-            if message == 'stopped':
-                stopped = True
-                break
-        if not stopped:
-            raise ServerStopError('Server thread failed to return ' + \
-                'stop event', {
-                    'server_id': self.id,
-                })
+            stopped = False
+            self.publish('force_stop')
+            for message in cache_db.subscribe(self.get_cache_key(),
+                    SUB_RESPONSE_TIMEOUT):
+                if message == 'stopped':
+                    stopped = True
+                    break
+            if not stopped:
+                raise ServerStopError('Server thread failed to return ' + \
+                    'stop event', {
+                        'server_id': self.id,
+                    })
 
-        if not silent:
-            Event(type=SERVERS_UPDATED)
-            LogEntry(message='Stopped server "%s".' % self.name)
+            if not silent:
+                Event(type=SERVERS_UPDATED)
+                LogEntry(message='Stopped server "%s".' % self.name)
+        finally:
+            cache_db.lock_release(self.get_cache_key('op_lock'))
 
     def restart(self, silent=False):
         if not self.status:
@@ -1043,16 +1193,19 @@ class Server(Config):
                     }
         self.update_clients(clients)
 
-    def update_clients(self, clients):
-        if not self.status:
+    def update_clients(self, clients, force=False):
+        if not force and not self.status:
             return
+        # Openvpn will create an undef client while a client connects
+        clients.pop('UNDEF', None)
         self._update_clients_bandwidth(clients)
         client_count = len(self.clients)
         self.clients = clients
-        if client_count != len(clients):
+        if force or client_count != len(clients):
             for org in self.iter_orgs():
                 Event(type=USERS_UPDATED, resource_id=org.id)
-            Event(type=SERVERS_UPDATED)
+            if not force:
+                Event(type=SERVERS_UPDATED)
 
     @classmethod
     def sort_servers_cache(cls):
@@ -1085,6 +1238,9 @@ class Server(Config):
             path = os.path.join(app_server.data_path, SERVERS_DIR)
             if os.path.isdir(path):
                 for server_id in os.listdir(path):
+                    if not os.path.isfile(os.path.join(path, server_id,
+                            SERVER_CONF_NAME)):
+                        continue
                     if os.path.isfile(os.path.join(path, server_id,
                             NODE_SERVER)):
                         server_id += '_' + NODE_SERVER
