@@ -25,7 +25,6 @@ class User(Config):
     str_options = {'name', 'otp_secret', 'type'}
     default_options = {
         'name': 'undefined',
-        'type': CERT_CLIENT,
     }
     chmod_mode = 0600
     cached = True
@@ -77,9 +76,9 @@ class User(Config):
             'otp_secret': self.otp_secret,
         }
 
-    def _upgrade_0_10_4(self):
+    def _upgrade_0_10_9(self):
         if not self.type:
-            logger.debug('Upgrading user to v0.10.4... %r' % {
+            logger.debug('Upgrading user to v0.10.9... %r' % {
                 'org_id': self.org.id,
                 'user_id': self.id,
             })
@@ -100,12 +99,18 @@ class User(Config):
         self.commit()
         self._cert_create()
         self._clean_openssl()
+
         if self.type != CERT_CA:
             cache_db.set_add(self.org.get_cache_key('users'), self.id)
             self._add_cache_trie_key()
+
         self.org.sort_users_cache()
         if self.type == CERT_CLIENT:
             LogEntry(message='Created new user "%s".' % self.name)
+
+            for server in self.org.iter_servers():
+                server.update_ip_pool()
+
         Event(type=ORGS_UPDATED)
         Event(type=USERS_UPDATED, resource_id=self.org.id)
         Event(type=SERVERS_UPDATED)
@@ -425,6 +430,10 @@ class User(Config):
 
         self._clean_openssl()
 
+        if type == CERT_CLIENT:
+            for server in self.org.iter_servers():
+                server.update_ip_pool()
+
         Event(type=ORGS_UPDATED)
         Event(type=USERS_UPDATED, resource_id=self.org.id)
         if type == CERT_CLIENT:
