@@ -22,6 +22,7 @@ THREADED_STRESS_TEST = True
 UUID_RE = r'^[a-z0-9]+$'
 AUTH_HANDLERS = [
     ('PUT', '/auth'),
+    ('GET', '/auth'),
     ('GET', '/export'),
     ('GET', '/event'),
     ('GET', '/key/a1/a1.tar'),
@@ -60,7 +61,8 @@ def _log_request(method, endpoint, start_time):
     if endpoint.startswith('/event'):
         return
     response_time = int((time.time() - start_time) * 1000)
-    if endpoint.startswith('/auth') or endpoint.startswith('/k/'):
+    if endpoint.startswith('/auth') or endpoint.startswith('/key/') \
+            or endpoint.startswith('/k/') or endpoint.startswith('/ku/'):
         if response_time > 900:
             color = '\033[92m'
         elif response_time > 700:
@@ -270,17 +272,6 @@ class Auth(SessionTestCase):
         })
         self.assertEqual(response.status_code, 200)
 
-        data = response.json()
-        self.assertIn('username', data)
-        self.assertEqual(data['username'], USERNAME)
-
-
-        response = self.session.post('/auth/token', json_data={
-            'username': USERNAME,
-            'password': TEST_PASSWORD,
-        })
-        self.assertEqual(response.status_code, 200)
-
 
         response = self.session.put('/auth', json_data={
             'username': USERNAME,
@@ -288,10 +279,6 @@ class Auth(SessionTestCase):
         })
         self.assertEqual(response.status_code, 200)
 
-        data = response.json()
-        self.assertIn('username', data)
-        self.assertEqual(data['username'], USERNAME)
-
     @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
     def test_auth_session_get(self):
         response = requests.get('/auth/session')
@@ -300,8 +287,6 @@ class Auth(SessionTestCase):
         self.assertEqual(response.status_code, 200)
         self.assertIn('authenticated', data)
         self.assertFalse(data['authenticated'])
-        self.assertIn('username', data)
-        self.assertEqual(data['username'], USERNAME)
 
     @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
     def test_auth_session_delete(self):
@@ -337,22 +322,21 @@ class Auth(SessionTestCase):
 
     @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
     def test_auth_post_error(self):
-        for endpoint in ('/auth/session', '/auth/token'):
-            for username, password in (
-                        ('admin', 'test'),
-                        ('test', 'admin'),
-                        ('test', 'test'),
-                    ):
-                response = requests.post(endpoint, json_data={
-                    'username': username,
-                    'password': password,
-                })
-                self.assertEqual(response.status_code, 401)
+        for username, password in (
+                    ('admin', 'test'),
+                    ('test', 'admin'),
+                    ('test', 'test'),
+                ):
+            response = requests.post('/auth/session', json_data={
+                'username': username,
+                'password': password,
+            })
+            self.assertEqual(response.status_code, 401)
 
-                data = response.json()
-                self.assertIn('error', data)
-                self.assertEqual(data['error'], 'auth_invalid')
-                self.assertIn('error_msg', data)
+            data = response.json()
+            self.assertIn('error', data)
+            self.assertEqual(data['error'], 'auth_invalid')
+            self.assertIn('error_msg', data)
 
     @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
     def test_auth_handlers(self):
@@ -360,53 +344,6 @@ class Auth(SessionTestCase):
             response = getattr(requests, method.lower())(endpoint)
             self.assertEqual(response.status_code, 401)
 
-    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
-    def test_auth_token_get_post_delete(self):
-        response = requests.get('/auth/session')
-        self.assertEqual(response.status_code, 200)
-
-        data = response.json()
-        self.assertIn('authenticated', data)
-        self.assertFalse(data['authenticated'])
-
-
-        response = requests.post('/auth/token', json_data={
-            'username': USERNAME,
-            'password': PASSWORD,
-        })
-        self.assertEqual(response.status_code, 200)
-
-        data = response.json()
-        self.assertIn('auth_token', data)
-        self.assertRegexpMatches(data['auth_token'], UUID_RE)
-        auth_token = data['auth_token']
-
-
-        response = requests.get('/auth/session', headers={
-            'Auth-Token': auth_token,
-        })
-        self.assertEqual(response.status_code, 200)
-        data = response.json()
-        self.assertIn('authenticated', data)
-        self.assertTrue(data['authenticated'])
-
-
-        response = requests.delete('/auth/token/%s' % auth_token)
-        self.assertEqual(response.status_code, 200)
-
-        data = response.json()
-        self.assertEqual(data, {})
-
-
-        response = requests.get('/auth/session', headers={
-            'Auth-Token': auth_token,
-        })
-        self.assertEqual(response.status_code, 200)
-
-        data = response.json()
-        self.assertIn('authenticated', data)
-        self.assertFalse(data['authenticated'])
-
 
 class Data(SessionTestCase):
     @unittest.skipUnless(ENABLE_EXTENDED_TESTS, 'Skipping test')
@@ -453,6 +390,7 @@ class Key(SessionTestCase):
         self.assertIn('id', data)
         self.assertIn('key_url', data)
         self.assertIn('view_url', data)
+        self.assertIn('uri_url', data)
 
         exp = r'^/key/[a-z0-9]+.tar$'
         self.assertRegexpMatches(data['key_url'], exp)
@@ -460,6 +398,9 @@ class Key(SessionTestCase):
         exp = r'^/k/[a-zA-Z0-9]+$'
         self.assertRegexpMatches(data['view_url'], exp)
 
+        exp = r'^/ku/[a-zA-Z0-9]+$'
+        self.assertRegexpMatches(data['uri_url'], exp)
+
 
         response = self.session.get(data['key_url'])
         self.assertEqual(response.status_code, 200)
@@ -478,7 +419,6 @@ class Key(SessionTestCase):
         content_type = response.headers['content-type']
         self.assertEqual(content_type, 'text/html; charset=utf-8')
 
-
         start_index = response.text.find('<h4 class="key-title">') + 22
         end_index = response.text.find('</h4>', start_index)
         self.assertNotEqual(start_index, -1)
@@ -487,7 +427,6 @@ class Key(SessionTestCase):
         self.assertEqual('%s - %s' % (TEST_ORG_NAME, TEST_USER_NAME),
             key_title)
 
-
         start_index = response.text.find(
             '<input id="key" type="text" readonly value="') + 44
         end_index = response.text.find('">', start_index)
@@ -497,7 +436,6 @@ class Key(SessionTestCase):
         exp = r'^[A-Z0-9]+$'
         self.assertRegexpMatches(otp_secret, exp)
 
-
         start_index = response.text.find('<a title="Download Key" href="') + 30
         end_index = response.text.find('">', start_index)
         self.assertNotEqual(start_index, -1)
@@ -505,7 +443,6 @@ class Key(SessionTestCase):
         key_url = response.text[start_index:end_index]
         self.assertEqual(key_url, data['key_url'])
 
-
         start_index = response.text.find(
             '<a class="sm" title="Download Mobile Key" href="') + 48
         end_index = response.text.find('">', start_index)
@@ -516,7 +453,6 @@ class Key(SessionTestCase):
         exp = r'^/key/[a-z0-9]+\.ovpn$'
         self.assertRegexpMatches(conf_url, exp)
 
-
         start_index = response.text.find("text: 'otpauth://totp/") + 7
         end_index = response.text.find("',", start_index)
         self.assertNotEqual(start_index, -1)
@@ -527,7 +463,6 @@ class Key(SessionTestCase):
             TEST_USER_NAME, TEST_ORG_NAME, otp_secret)
         self.assertRegexpMatches(otp_key, exp)
 
-
         start_index = response.text.find("xmlhttp.open('DELETE', '") + 24
         end_index = response.text.find("', false);", start_index)
         self.assertNotEqual(start_index, -1)
@@ -536,6 +471,13 @@ class Key(SessionTestCase):
         self.assertEqual(data['view_url'], delete_link)
 
 
+        response = self.session.get(data['uri_url'])
+        self.assertEqual(response.status_code, 200)
+
+        content_type = response.headers['content-type']
+        self.assertEqual(content_type, 'application/json')
+
+
         response = self.session.delete(data['view_url'])
         self.assertEqual(response.status_code, 200)
 
@@ -653,6 +595,7 @@ class Server(SessionTestCase):
             'port': 12345,
             'protocol': 'udp',
             'dh_param_bits': 1024,
+            'mode': 'local_traffic',
             'local_networks': ['10.0.0.0/24'],
             'dns_servers': ['8.8.4.4'],
             'public_address': '8.8.8.8',
@@ -677,6 +620,8 @@ class Server(SessionTestCase):
         self.assertEqual(data['protocol'], 'udp')
         self.assertIn('dh_param_bits', data)
         self.assertEqual(data['dh_param_bits'], 1024)
+        self.assertIn('mode', data)
+        self.assertEqual(data['mode'], 'local_traffic')
         self.assertIn('local_networks', data)
         self.assertEqual(data['local_networks'], ['10.0.0.0/24'])
         self.assertIn('dns_servers', data)
@@ -712,6 +657,8 @@ class Server(SessionTestCase):
         self.assertEqual(data['protocol'], 'udp')
         self.assertIn('dh_param_bits', data)
         self.assertEqual(data['dh_param_bits'], 1024)
+        self.assertIn('mode', data)
+        self.assertEqual(data['mode'], 'local_traffic')
         self.assertIn('local_networks', data)
         self.assertEqual(data['local_networks'], ['10.0.0.0/24'])
         self.assertIn('dns_servers', data)
@@ -741,6 +688,7 @@ class Server(SessionTestCase):
             self.assertIn('port', server)
             self.assertIn('protocol', server)
             self.assertIn('dh_param_bits', server)
+            self.assertIn('mode', server)
             self.assertIn('local_networks', server)
             self.assertIn('dns_servers', server)
             self.assertIn('public_address', server)
@@ -756,6 +704,7 @@ class Server(SessionTestCase):
                 self.assertEqual(server['port'], 12345)
                 self.assertEqual(server['protocol'], 'udp')
                 self.assertEqual(server['dh_param_bits'], 1024)
+                self.assertEqual(server['mode'], 'local_traffic')
                 self.assertEqual(server['local_networks'], ['10.0.0.0/24'])
                 self.assertEqual(server['dns_servers'], ['8.8.4.4'])
                 self.assertEqual(server['public_address'], '8.8.8.8')
@@ -1059,6 +1008,18 @@ class Server(SessionTestCase):
         self.assertIn('error_msg', data)
 
 
+        response = self.session.post('/server', json_data={
+            'name': TEST_SERVER_NAME + '_test',
+            'mode': 'test',
+        })
+        self.assertEqual(response.status_code, 400)
+
+        data = response.json()
+        self.assertIn('error', data)
+        self.assertEqual(data['error'], 'mode_invalid')
+        self.assertIn('error_msg', data)
+
+
         response = self.session.post('/server', json_data={
             'name': TEST_SERVER_NAME + '_test',
             'dns_servers': ['8.8.8.a'],
@@ -1098,6 +1059,7 @@ class Status(SessionTestCase):
         self.assertIn('server_version', data)
         self.assertIn('public_ip', data)
         self.assertIn('local_networks', data)
+        self.assertIn('notification', data)
 
 
 class User(SessionTestCase):
