@@ -29,10 +29,27 @@ define([
     return key;
   }
 
+  var authGet = function(request) {
+    request.response({
+      username: demoData.auth.username,
+      token: demoData.auth.token,
+      secret: demoData.auth.secret
+    });
+  };
+  routes['GET=/auth'] = authGet;
+
   var authPut = function(request) {
     demoData.auth.username = request.data.username;
     demoData.auth.password = request.data.password;
-    request.response({});
+    if (!request.data.token) {
+      demoData.auth.token = uuid();
+      demoData.auth.secret = uuid();
+    }
+    request.response({
+      username: demoData.auth.username,
+      token: demoData.auth.token,
+      secret: demoData.auth.secret
+    });
   };
   routes['PUT=/auth'] = authPut;
 
@@ -63,6 +80,43 @@ define([
   };
   routes['DELETE=/auth/session'] = authSessionDelete;
 
+  var subscriptionGet = function(request) {
+    request.response(demoData.subscription);
+  };
+  routes['GET=/subscription'] = subscriptionGet;
+
+  var subscriptionStateGet = function(request) {
+    request.response({active: demoData.subscription.active});
+  };
+  routes['GET=/subscription/state'] = subscriptionStateGet;
+
+  var subscriptionPost = function(request) {
+    demoData.subscription.license = true;
+    demoData.subscription.active = true;
+    event('subscription_active');
+    request.response(demoData.subscription);
+  };
+  routes['POST=/subscription'] = subscriptionPost;
+
+  var subscriptionPut = function(request) {
+    if (request.data.cancel) {
+      demoData.subscription.cancel_at_period_end = true;
+    }
+    else {
+      demoData.subscription.cancel_at_period_end = false;
+    }
+    request.response(demoData.subscription);
+  };
+  routes['PUT=/subscription'] = subscriptionPut;
+
+  var subscriptionDelete = function(request) {
+    demoData.subscription.license = false;
+    demoData.subscription.active = false;
+    event('subscription_inactive');
+    request.response(demoData.subscription);
+  };
+  routes['DELETE=/subscription'] = subscriptionDelete;
+
   var event = function(type, resourceId) {
     demoData.events.push({
       id: uuid(),
@@ -243,6 +297,7 @@ define([
       port: request.data.port,
       protocol: request.data.protocol,
       dh_param_bits: request.data.dh_param_bits,
+      mode: request.data.mode,
       local_networks: request.data.local_networks,
       dns_servers: request.data.dns_servers,
       public_address: request.data.public_address,
@@ -269,6 +324,7 @@ define([
     demoData.servers[serverId].port = request.data.port;
     demoData.servers[serverId].protocol = request.data.protocol;
     demoData.servers[serverId].dh_param_bits = request.data.dh_param_bits;
+    demoData.servers[serverId].mode = request.data.mode;
     demoData.servers[serverId].local_networks = request.data.local_networks;
     demoData.servers[serverId].dns_servers = request.data.dns_servers;
     demoData.servers[serverId].public_address = request.data.public_address;
@@ -408,7 +464,8 @@ define([
         '10.20.1.0/24',
         '10.30.2.0/24',
         '192.168.1.0/24'
-      ]
+      ],
+      notification: ''
     });
   };
   routes['GET=/status'] = statusGet;
@@ -469,7 +526,18 @@ define([
       status: false,
       otp_auth: true,
       otp_secret: secretKey(),
-      servers: []
+      servers: [{
+        id: '5310c28eacaeb3bd2a172c9db9ee1379',
+        name: 'server1',
+        status: false,
+        local_address: '10.54.0.12',
+        remote_address: '10.54.0.13',
+        real_address: null,
+        virt_address: null,
+        bytes_received: null,
+        bytes_sent: null,
+        connected_since: null
+      }]
     };
 
     event('users_updated', orgId);
@@ -594,7 +662,9 @@ define([
             status = 'error';
             ajaxRequest.error(jqXHR, status, null);
           }
-          ajaxRequest.complete(jqXHR, status);
+          if (ajaxRequest.complete) {
+            ajaxRequest.complete(jqXHR, status);
+          }
         }, responseDelay);
       }
     };
