@@ -77,6 +77,7 @@ require.config({
     qrcode: 'vendor/qrcode/qrcode',
     rickshaw: 'vendor/rickshaw/rickshaw',
     select: 'vendor/select/select',
+    sjcl: 'vendor/sjcl/sjcl.min',
     text: 'vendor/requireText/text',
     underscore: 'vendor/underscore/underscore',
     initialize: 'init/testing'
@@ -94,6 +95,7 @@ require.config({
     bootstrap: ['jquery'],
     d3: {exports: 'd3'},
     rickshaw: {deps: ['d3'], exports: 'Rickshaw'},
+    sjcl: {exports: 'sjcl'},
     qrcode: {exports: 'QRCode'}
   }
 });
@@ -132,12 +134,13 @@ require([
   'jquery',
   'underscore',
   'backbone',
+  'models/subscription',
   'collections/event',
   'views/header',
   'routers/main',
   'initialize'
-], function($, _, Backbone, EventCollection, HeaderView, mainRouter,
-    initialize) {
+], function($, _, Backbone, SubscriptionModel, EventCollection, HeaderView,
+    mainRouter, initialize) {
   'use strict';
 
   initialize();
@@ -153,7 +156,7 @@ require([
     return days + 'd ' + hours + 'h ' + minutes + 'm ' + seconds + 's';
   };
 
-  window.formatTime = function(time, short) {
+  window.formatTime = function(time, type) {
     var abbrev = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
       'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
     var date = new Date(0);
@@ -180,7 +183,7 @@ require([
     if (minutes.length < 2) {
       minutes = '0' + minutes;
     }
-    if (short) {
+    if (type === 'short') {
       if (curDate.getDate() === date.getDate() &&
           curDate.getMonth() === date.getMonth() &&
           curDate.getFullYear() === date.getFullYear()) {
@@ -193,6 +196,9 @@ require([
         }
       }
     }
+    else if (type === 'date') {
+      time = month + ' ' + day;
+    }
     else {
       time = hours + ':' + minutes + ' ' + meridiem + ' - ' +
         month + ' ' + day + ' ' + year;
@@ -227,6 +233,15 @@ require([
     return bytes;
   };
 
+  $.getCachedScript = function(url, options) {
+    options = $.extend(options || {}, {
+      dataType: 'script',
+      cache: true,
+      url: url
+    });
+    return $.ajax(options);
+  };
+
   $.fn.roll = function(timeout, complete) {
     var timer;
 
@@ -262,11 +277,28 @@ require([
 
   $(document).on('dblclick mousedown', '.no-select', false);
 
-  window.events = new EventCollection();
-  window.events.start();
+  var init = function() {
+    window.events = new EventCollection();
+    window.events.start();
 
-  var headerView = new HeaderView();
-  $('body').prepend(headerView.render().el);
+    var headerView = new HeaderView();
+    $('body').prepend(headerView.render().el);
 
-  mainRouter.initialize();
+    mainRouter.initialize();
+  };
+
+  var model = new SubscriptionModel();
+  model.fetch({
+    url: '/subscription/state',
+    success: function(model) {
+      window.enterprise = model.get('active');
+      if (window.enterprise) {
+        $('body').addClass('enterprise');
+      }
+      init();
+    },
+    error: function() {
+      init();
+    }
+  });
 });
