@@ -58,8 +58,8 @@ define([
         }.bind(this)
       });
     },
-    onClickInput: function() {
-      this.$('input').select();
+    onClickInput: function(evt) {
+      this.$(evt.target).select();
     }
   });
 
