@@ -44,35 +44,34 @@ define([
       });
       this.$('.local-network input').select2(
         'val', this.model.get('local_networks'));
+      this.$('.network .label').tooltip();
+      this.updateMaxHosts();
     },
     getServerMode: function() {
       return this.$('.server-mode select').val();
     },
     setServerMode: function(mode) {
-      if (mode === 'lan') {
-        this.$('.server-mode select').val('lan');
-        this.$('.local-network').slideDown(250);
-        this.$('.otp-auth-toggle').slideUp(250, function() {
-          this.$('.otp-auth-toggle').appendTo('.left');
-          this.$('.otp-auth-toggle').show();
-        }.bind(this));
+      if (mode === 'local_traffic') {
+        if (!this.$('.otp-auth-toggle').parent().hasClass('left')) {
+          this.$('.local-network').slideDown(250);
+          this.$('.otp-auth-toggle').slideUp(250, function() {
+            this.$('.otp-auth-toggle').appendTo('.left');
+            this.$('.otp-auth-toggle').show();
+          }.bind(this));
+        }
       }
       else {
-        this.$('.server-mode select').val('all');
-        this.$('.local-network').slideUp(250);
-        this.$('.otp-auth-toggle').slideUp(250, function() {
-          this.$('.otp-auth-toggle').appendTo('.right');
-          this.$('.otp-auth-toggle').show();
-        }.bind(this));
+        if (!this.$('.otp-auth-toggle').parent().hasClass('right')) {
+          this.$('.local-network').slideUp(250);
+          this.$('.otp-auth-toggle').slideUp(250, function() {
+            this.$('.otp-auth-toggle').appendTo('.right');
+            this.$('.otp-auth-toggle').show();
+          }.bind(this));
+        }
       }
     },
     onServerMode: function() {
-      if (this.getServerMode() === 'lan') {
-        this.setServerMode('lan');
-      }
-      else {
-        this.setServerMode('all');
-      }
+      this.setServerMode(this.getServerMode());
     },
     onDhParamBits: function(evt) {
       var val = $(evt.target).val();
@@ -120,6 +119,42 @@ define([
     onDebugSelect: function() {
       this.setDebugSelect(!this.getDebugSelect());
     },
+    onInputChange: function(evt) {
+      if ($(evt.target).parent().hasClass('network')) {
+        this.updateMaxHosts();
+      }
+    },
+    updateMaxHosts: function() {
+      var value = this.$('.network input').val().split('/');
+      var maxHosts = {
+        8: '4m',
+        9: '2m',
+        10: '1m',
+        11: '524k',
+        12: '262k',
+        13: '131k',
+        14: '65k',
+        15: '32k',
+        16: '16k',
+        17: '8k',
+        18: '4k',
+        19: '2k',
+        20: '1k',
+        21: '512',
+        22: '256',
+        23: '128',
+        24: '64',
+      };
+      if (value.length === 2) {
+        var max = maxHosts[value[1]];
+        if (max) {
+          this.$('.network .label').text(max + ' Users');
+          this.$('.network .label').show();
+          return;
+        }
+      }
+      this.$('.network .label').hide();
+    },
     onOk: function() {
       var i;
       var name = this.$('.name input').val();
@@ -128,6 +163,7 @@ define([
       var port = this.$('.port input').val();
       var protocol = this.$('.protocol select').val();
       var dhParamBits = parseInt(this.$('.dh-param-bits select').val(), 10);
+      var mode = this.$('.server-mode select').val();
       var publicAddress = this.$('.public-address input').val();
       var dnsServers = [];
       var dnsServersTemp = this.$('.dns-servers input').val().split(',');
@@ -169,7 +205,7 @@ define([
           '.public-address');
         return;
       }
-      if (this.getServerMode() === 'lan') {
+      if (this.getServerMode() === 'local_traffic') {
         localNetworks = this.$('.local-network input').select2('val');
         if (!localNetworks) {
           this.setAlert('danger', 'Local network can not be empty.',
@@ -186,6 +222,7 @@ define([
         'port': port,
         'protocol': protocol,
         'dh_param_bits': dhParamBits,
+        'mode': mode,
         'local_networks': localNetworks,
         'dns_servers': dnsServers,
         'public_address': publicAddress,
