@@ -6,64 +6,64 @@ define([
   'views/modalRenameUser',
   'views/modalKeyLink',
   'views/modalOtpAuth',
+  'views/userServersList',
   'text!templates/usersListItem.html'
 ], function($, _, Backbone, KeyModel, ModalRenameUserView, ModalKeyLinkView,
-    ModalOtpAuthView, usersListItemTemplate) {
+    ModalOtpAuthView, UserServersListView, usersListItemTemplate) {
   'use strict';
   var UsersListItemView = Backbone.View.extend({
-    className: 'user',
     template: _.template(usersListItemTemplate),
     events: {
       'click .selector': 'onSelect',
       'click .user-name': 'onRename',
       'click .get-key-link': 'onGetKeyLink',
-      'click .get-otp-auth': 'onGetOtpAuth'
+      'click .get-otp-auth': 'onGetOtpAuth',
+      'click .toggle-servers ': 'onToggleServers'
+    },
+    initialize: function() {
+      this.serverList = new UserServersListView({
+        models: this.model.get('servers')
+      });
     },
     getVirtAddresses: function() {
-      var serverId;
+      var i;
       var servers = this.model.get('servers');
       var virtAddresses = [];
 
-      for (serverId in servers) {
-        virtAddresses.push(servers[serverId].virt_address);
+      for (i = 0; i < servers.length; i++) {
+        if (servers[i].virt_address) {
+          virtAddresses.push(servers[i].virt_address);
+        }
       }
 
       return virtAddresses;
     },
     getBytesReceived: function() {
-      var serverId;
+      var i;
       var servers = this.model.get('servers');
       var bytesReceived = 0;
 
-      for (serverId in servers) {
-        bytesReceived += servers[serverId].bytes_received;
+      for (i = 0; i < servers.length; i++) {
+        if (servers[i].bytes_received) {
+          bytesReceived += servers[i].bytes_received;
+        }
       }
 
       return window.formatSize(bytesReceived);
     },
     getBytesSent: function() {
-      var serverId;
+      var i;
       var servers = this.model.get('servers');
       var bytesSent = 0;
 
-      for (serverId in servers) {
-        bytesSent += servers[serverId].bytes_sent;
+      for (i = 0; i < servers.length; i++) {
+        if (servers[i].bytes_sent) {
+          bytesSent += servers[i].bytes_sent;
+        }
       }
 
       return window.formatSize(bytesSent);
     },
-    _getStatusTooltip: function() {
-      var bytesSent = this.getBytesSent();
-      var bytesReceived = this.getBytesReceived();
-      var virtAddresses = this.getVirtAddresses();
-      if (!virtAddresses.length) {
-        return '';
-      }
-      var tooltipText = 'IP Address: ' + virtAddresses.join(', ');
-      tooltipText += '\nData Sent: ' + bytesSent;
-      tooltipText += '\nData Received: ' + bytesReceived;
-      return tooltipText;
-    },
     _getDownloadTooltip: function() {
       if (this.model.get('has_key')) {
         return 'Click to download key';
@@ -86,47 +86,40 @@ define([
     },
     _getKeyLinkTooltip: function() {
       if (this.model.get('has_key')) {
-        return 'Click to generate temporary key url that is downloadable ' +
-          'without authenticating';
+        return 'Get key links';
       }
       else {
-        return 'Organization must be attached to server to generate key link';
+        return 'Organization must be attached to server to get key links';
       }
     },
     render: function() {
       this.$el.html(this.template(_.extend(
         {
-          'status_tooltip': this._getStatusTooltip(),
           'download_tooltip': this._getDownloadTooltip(),
           'key_link': this._getKeyLink(),
           'key_link_tooltip': this._getKeyLinkTooltip()
         }, this.model.toJSON())));
       this.$('[data-toggle="tooltip"]').tooltip();
+      this.$el.append(this.serverList.render().el);
       return this;
     },
     update: function() {
       this.$('.user-name').text(this.model.get('name'));
       if (this.model.get('status')) {
-        if (!this.$('.status-icon').hasClass('online')) {
-          this.$('.status-icon').removeClass('offline');
-          this.$('.status-icon').addClass('online');
-          this.$('.status-text').text('Online');
+        if (!this.$('.user .status-icon').hasClass('online')) {
+          this.$('.user .status-icon').removeClass('offline');
+          this.$('.user .status-icon').addClass('online');
+          this.$('.user .status-text').text('Online');
         }
       }
       else {
-        if (!this.$('.status-icon').hasClass('offline')) {
-          this.$('.status-icon').removeClass('online');
-          this.$('.status-icon').addClass('offline');
-          this.$('.status-text').text('Offline');
+        if (!this.$('.user .status-icon').hasClass('offline')) {
+          this.$('.user .status-icon').removeClass('online');
+          this.$('.user .status-icon').addClass('offline');
+          this.$('.user .status-text').text('Offline');
         }
       }
 
-      this.$('.status-container').tooltip('destroy');
-      this.$('.status-container').attr('title', this._getStatusTooltip());
-      this.$('.status-container').attr('data-original-title',
-        this._getStatusTooltip());
-      this.$('.status-container').tooltip();
-
       this.$('.download-key').tooltip('destroy');
       this.$('.download-key').attr('title', this._getDownloadTooltip());
       this.$('.download-key').attr('data-original-title',
@@ -147,6 +140,8 @@ define([
         this.$('.right-container').addClass('no-otp-auth');
         this.$('.get-otp-auth').addClass('no-otp-auth');
       }
+
+      this.serverList.update(this.model.get('servers'));
     },
     getSelect: function() {
       return this.$('.selector').hasClass('selected');
@@ -189,6 +184,26 @@ define([
         model: this.model
       });
       this.addView(modal);
+    },
+    onToggleServers: function() {
+      var tooltipText;
+      if (this.$('.toggle-servers').hasClass('glyphicon-chevron-down')) {
+        this.$('.toggle-servers').removeClass('glyphicon-chevron-down');
+        this.$('.toggle-servers').addClass('glyphicon-chevron-up');
+        this.$('.user-servers').slideDown(200);
+        tooltipText = 'Hide additional user information';
+      }
+      else {
+        this.$('.toggle-servers').removeClass('glyphicon-chevron-up');
+        this.$('.toggle-servers').addClass('glyphicon-chevron-down');
+        this.$('.user-servers').slideUp(200);
+        tooltipText = 'Show additional user information';
+      }
+
+      this.$('.toggle-servers').tooltip('destroy');
+      this.$('.toggle-servers').attr('title', tooltipText);
+      this.$('.toggle-servers').attr('data-original-title', tooltipText);
+      this.$('.toggle-servers').tooltip();
     }
   });
 
